@@ -52,6 +52,8 @@ using namespace KJS;
 using DOM::DOMException;
 using DOM::DOMString;
 using DOM::NodeFilter;
+using DOM::NodeImpl;
+using DOM::DocumentImpl;
 
 // -------------------------------------------------------------------------
 /* Source for DOMNodeProtoTable. Use "make hashtables" to regenerate.
@@ -82,13 +84,76 @@ IMPLEMENT_PROTOTYPE(DOMNodeProto,DOMNodeProtoFunc)
 const ClassInfo DOMNode::info = { "Node", 0, &DOMNodeTable, 0 };
 
 DOMNode::DOMNode(ExecState *exec, const DOM::Node &n)
-  : DOMObject(DOMNodeProto::self(exec)), node(n)
+  : node(n)
+{
+  setPrototype(DOMNodeProto::self(exec));
+}
+
+DOMNode::DOMNode(const DOM::Node &n)
+  : node(n)
 {
 }
 
-DOMNode::DOMNode(const Object &proto, const DOM::Node &n)
-  : DOMObject(proto), node(n)
+DOMNode::~DOMNode()
 {
+  NodeImpl *n = node.handle();
+  if (n) {
+    ScriptInterpreter::forgetDOMNodeForDocument(n->getDocument(), n);
+  }
+}
+
+void DOMNode::mark()
+{
+  assert(!marked());
+
+
+  // Nodes in the document are kept alive by ScriptInterpreter::mark,
+  // so we have no special responsibilities and can just call the base class here.
+  if (node.isNull() || node.handle()->inDocument()) {
+    DOMObject::mark();
+    return;
+  }
+  
+  // This is a node outside the document, so find the root of the tree it is in,
+  // and start marking from there.
+  NodeImpl *root = node.handle();
+  for (NodeImpl *current = node.handle(); current; current = current->parentNode()) {
+    root = current;
+  }
+
+  static QPtrDict<NodeImpl> markingRoots;
+
+  // If we're already marking this tree, then we can simply mark this wrapper
+  // by calling the base class; our caller is iterating the tree.
+  if (markingRoots.find(root)) {
+    DOMObject::mark();
+    return;
+  }
+
+  DocumentImpl *document = node.handle()->getDocument();
+
+  // Mark the whole tree; use the global set of roots to avoid reentering.
+  markingRoots.insert(root, root);
+  for (NodeImpl *nodeToMark = root; nodeToMark; nodeToMark = nodeToMark->traverseNextNode()) {
+    DOMNode *wrapper = ScriptInterpreter::getDOMNodeForDocument(document, nodeToMark);
+    if (wrapper) {
+      if (!wrapper->marked())
+        wrapper->mark();
+    } else if (nodeToMark == node.handle()) {
+      // This is the case where the map from the document to wrappers has
+      // been cleared out, but a wrapper is being marked. For now, we'll
+      // let the rest of the tree of wrappers get collected, because we have
+      // no good way of finding them. Later we should test behavior of other
+      // browsers and see if we need to preserve other wrappers in this case.
+      if (!marked())
+        mark();
+    }
+  }
+  markingRoots.remove(root);
+
+  // Double check that we actually ended up marked. This assert caught problems in the past.
+  assert(marked());
+
 }
 
 bool DOMNode::toBoolean(ExecState *) const
@@ -831,10 +896,15 @@ const ClassInfo DOMDocument::info = { "Document", &DOMNode::info, &DOMDocumentTa
 */
 
 DOMDocument::DOMDocument(ExecState *exec, const DOM::Document &d)
-  : DOMNode(DOMDocumentProto::self(exec), d) { }
+  : DOMNode(d) 
+{ 
+  setPrototype(DOMDocumentProto::self(exec));
+}
 
-DOMDocument::DOMDocument(const Object &proto, const DOM::Document &d)
-  : DOMNode(proto, d) { }
+DOMDocument::DOMDocument(const DOM::Document &d)
+  : DOMNode(d) 
+{ 
+}
 
 DOMDocument::~DOMDocument()
 {
@@ -1052,10 +1122,15 @@ const ClassInfo DOMElement::info = { "Element", &DOMNode::info, &DOMElementTable
 @end
 */
 DOMElement::DOMElement(ExecState *exec, const DOM::Element &e)
-  : DOMNode(DOMElementProto::self(exec), e) { }
+  : DOMNode(e) 
+{
+  setPrototype(DOMElementProto::self(exec));
+}
 
-DOMElement::DOMElement(const Object &proto, const DOM::Element &e)
-  : DOMNode(proto, e) { }
+DOMElement::DOMElement(const DOM::Element &e)
+  : DOMNode(e) 
+{ 
+}
 
 Value DOMElement::tryGet(ExecState *exec, const Identifier &propertyName) const
 {
@@ -1183,7 +1258,10 @@ IMPLEMENT_PROTOTYPE(DOMDOMImplementationProto,DOMDOMImplementationProtoFunc)
 const ClassInfo DOMDOMImplementation::info = { "DOMImplementation", 0, 0, 0 };
 
 DOMDOMImplementation::DOMDOMImplementation(ExecState *exec, const DOM::DOMImplementation &i)
-  : DOMObject(DOMDOMImplementationProto::self(exec)), implementation(i) { }
+  : implementation(i) 
+{ 
+  setPrototype(DOMDOMImplementationProto::self(exec));
+}
 
 DOMDOMImplementation::~DOMDOMImplementation()
 {
@@ -1283,7 +1361,10 @@ IMPLEMENT_PROTOTYPE(DOMNamedNodeMapProto,DOMNamedNodeMapProtoFunc)
 const ClassInfo DOMNamedNodeMap::info = { "NamedNodeMap", 0, 0, 0 };
 
 DOMNamedNodeMap::DOMNamedNodeMap(ExecState *exec, const DOM::NamedNodeMap &m)
-  : DOMObject(DOMNamedNodeMapProto::self(exec)), map(m) { }
+  : map(m) 
+{ 
+  setPrototype(DOMNamedNodeMapProto::self(exec));
+}
 
 DOMNamedNodeMap::~DOMNamedNodeMap()
 {
@@ -1489,13 +1570,13 @@ bool KJS::checkNodeSecurity(ExecState *exec, const DOM::Node& n)
 
 Value KJS::getDOMNode(ExecState *exec, const DOM::Node &n)
 {
-  DOMObject *ret = 0;
+  DOMNode *ret = 0;
   if (n.isNull())
     return Null();
   ScriptInterpreter* interp = static_cast<ScriptInterpreter *>(exec->dynamicInterpreter());
-  DOM::NodeImpl *doc = n.ownerDocument().handle();
+  DocumentImpl *doc = n.handle()->getDocument();
 
-  if ((ret = interp->getDOMObjectForDocument(static_cast<DOM::DocumentImpl *>(doc), n.handle())))
+  if ((ret = interp->getDOMNodeForDocument(doc, n.handle())))
     return Value(ret);
 
   switch (n.nodeType()) {
@@ -1540,7 +1621,7 @@ Value KJS::getDOMNode(ExecState *exec, const DOM::Node &n)
       ret = new DOMNode(exec, n);
   }
 
-  interp->putDOMObjectForDocument(static_cast<DOM::DocumentImpl *>(doc), n.handle(), ret);
+  interp->putDOMNodeForDocument(doc, n.handle(), ret);
 
   return Value(ret);
 }
@@ -1745,7 +1826,7 @@ Object KJS::getDOMExceptionConstructor(ExecState *exec)
 // for constructs like document.forms.<name>[1],
 // so it shouldn't be a problem that it's storing all the nodes (with the same name). (David)
 DOMNamedNodesCollection::DOMNamedNodesCollection(ExecState *, const QValueList<DOM::Node>& nodes )
-  : DOMObject(), m_nodes(nodes)
+  : m_nodes(nodes)
 {
   // Maybe we should ref (and deref in the dtor) the nodes, though ?
 }
@@ -1808,10 +1889,15 @@ IMPLEMENT_PROTOFUNC(DOMCharacterDataProtoFunc)
 IMPLEMENT_PROTOTYPE_WITH_PARENT(DOMCharacterDataProto,DOMCharacterDataProtoFunc, DOMNodeProto)
 
 DOMCharacterData::DOMCharacterData(ExecState *exec, const DOM::CharacterData &d)
- : DOMNode(DOMCharacterDataProto::self(exec), d) {}
+ : DOMNode(d) 
+{
+  setPrototype(DOMCharacterDataProto::self(exec));
+}
 
-DOMCharacterData::DOMCharacterData(const Object &proto, const DOM::CharacterData &d)
- : DOMNode(proto, d) {}
+DOMCharacterData::DOMCharacterData(const DOM::CharacterData &d)
+ : DOMNode(d) 
+{
+}
 
 Value DOMCharacterData::tryGet(ExecState *exec, const Identifier &p) const
 {
@@ -1897,7 +1983,10 @@ IMPLEMENT_PROTOFUNC(DOMTextProtoFunc)
 IMPLEMENT_PROTOTYPE_WITH_PARENT(DOMTextProto,DOMTextProtoFunc,DOMCharacterDataProto)
 
 DOMText::DOMText(ExecState *exec, const DOM::Text &t)
-  : DOMCharacterData(DOMTextProto::self(exec), t) { }
+  : DOMCharacterData(t) 
+{ 
+  setPrototype(DOMTextProto::self(exec));
+}
 
 Value DOMText::tryGet(ExecState *exec, const Identifier &p) const
 {
