@@ -66,6 +66,7 @@
 
 using DOM::DocumentImpl;
 using DOM::DOMString;
+using DOM::ElementImpl;
 using DOM::Node;
 using DOM::Position;
 using khtml::TypingCommand;
@@ -292,6 +293,7 @@ const ClassInfo Window::info = { "Window", 0, &WindowTable, 0 };
   onselect	Window::Onselect	DontDelete
   onsubmit	Window::Onsubmit	DontDelete
   onunload	Window::Onunload	DontDelete
+  frameElement  Window::FrameElement    DontDelete|ReadOnly
 @end
 */
 IMPLEMENT_PROTOFUNC(WindowFunc)
@@ -830,6 +832,12 @@ Value Window::get(ExecState *exec, const Identifier &p) const
         return getListener(exec,DOM::EventImpl::UNLOAD_EVENT);
       else
         return Undefined();
+    case FrameElement:
+      if (DocumentImpl *doc = m_part->xmlDocImpl())
+        if (ElementImpl *fe = doc->ownerElement())
+          if (checkNodeSecurity(exec, fe))
+            return getDOMNode(exec, fe);
+      return Undefined();
     }
   }
 
