@@ -229,8 +229,7 @@ void XMLHttpRequest::mark()
 
 
 XMLHttpRequest::XMLHttpRequest(ExecState *exec, const DOM::Document &d)
-  : DOMObject(XMLHttpRequestProto::self(exec)),
-    qObject(new XMLHttpRequestQObject(this)),
+  : qObject(new XMLHttpRequestQObject(this)),
     doc(static_cast<DOM::DocumentImpl*>(d.handle())),
     async(true),
     job(0),
@@ -241,6 +240,7 @@ XMLHttpRequest::XMLHttpRequest(ExecState *exec, const DOM::Document &d)
     createdDocument(false),
     aborted(false)
 {
+  setPrototype(XMLHttpRequestProto::self(exec));
 }
 
 XMLHttpRequest::~XMLHttpRequest()
@@ -321,6 +321,13 @@ void XMLHttpRequest::open(const QString& _method, const KURL& _url, bool _async)
 
 void XMLHttpRequest::send(const QString& _body)
 {
+  if (!doc)
+    return;
+
+  // FIXME: Should this abort instead if we already have a job going?
+  if (job)
+    return;
+
   aborted = false;
 
 #if !APPLE_CHANGES
@@ -369,6 +376,8 @@ void XMLHttpRequest::send(const QString& _body)
   qObject->connect( job, SIGNAL(redirection(KIO::Job*, const KURL& ) ),
 		    SLOT( slotRedirection(KIO::Job*, const KURL&) ) );
 
+  addToRequestsByDocument();
+
 #ifdef APPLE_CHANGES
   KWQServeRequest(khtml::Cache::loader(), doc->docLoader(), job);
 #else 
@@ -378,7 +387,10 @@ void XMLHttpRequest::send(const QString& _body)
 
 void XMLHttpRequest::abort()
 {
-  if (job) {
+  bool hadJob = job;
+
+  if (hadJob) {
+    removeFromRequestsByDocument();
     job->kill();
     job = 0;
   }
@@ -388,7 +400,8 @@ void XMLHttpRequest::abort()
   }
   aborted = true;
 
-  gcUnprotect (this);
+  if (hadJob)
+    gcUnprotect(this);
 }
 
 void XMLHttpRequest::setRequestHeader(const QString& name, const QString &value)
@@ -523,6 +536,7 @@ void XMLHttpRequest::slotFinished(KIO::Job *)
     response += decoder->flush();
   }
 
+  removeFromRequestsByDocument();
   job = 0;
 
   changeState(Completed);
@@ -582,6 +596,50 @@ void XMLHttpRequest::slotData(KIO::Job*, const QByteArray &_data)
   }
 }
 
+QPtrDict< QPtrDict<XMLHttpRequest> > &XMLHttpRequest::requestsByDocument()
+{
+    static QPtrDict< QPtrDict<XMLHttpRequest> > dictionary;
+    return dictionary;
+}
+
+void XMLHttpRequest::addToRequestsByDocument()
+{
+  assert(doc);
+
+  QPtrDict<XMLHttpRequest> *requests = requestsByDocument().find(doc);
+  if (!requests) {
+    requests = new QPtrDict<XMLHttpRequest>;
+    requestsByDocument().insert(doc, requests);
+  }
+
+  assert(requests->find(this) == 0);
+  requests->insert(this, this);
+}
+
+void XMLHttpRequest::removeFromRequestsByDocument()
+{
+  assert(doc);
+
+  QPtrDict<XMLHttpRequest> *requests = requestsByDocument().find(doc);
+
+  // Since synchronous loads are not added to requestsByDocument(), we need to make sure we found the request.
+  if (!requests || !requests->find(this))
+    return;
+
+  requests->remove(this);
+
+  if (requests->isEmpty()) {
+    requestsByDocument().remove(doc);
+    delete requests;
+  }
+}
+
+void XMLHttpRequest::cancelRequests(DOM::DocumentImpl *d)
+{
+  while (QPtrDict<XMLHttpRequest> *requests = requestsByDocument().find(d))
+    QPtrDictIterator<XMLHttpRequest>(*requests).current()->abort();
+}
+
 Value XMLHttpRequestProtoFunc::tryCall(ExecState *exec, Object &thisObj, const List &args)
 {
   if (!thisObj.inherits(&XMLHttpRequest::info)) {
