@@ -491,6 +491,9 @@ protected:
         void setCachePolicy( KIO::CacheControl cachePolicy );
         void setShowAnimations( KHTMLSettings::KAnimationAdvice );
         void removeCachedObject( CachedObject*) const;
+		
+        void setLoadInProgress(bool);
+        bool loadInProgress() const { return m_loadInProgress; }
 
     private:
         bool needReload(const KURL &fullUrl);
@@ -506,6 +509,7 @@ protected:
         KHTMLSettings::KAnimationAdvice m_showAnimations : 2;
         KHTMLPart* m_part;
         DOM::DocumentImpl* m_doc;
+        bool m_loadInProgress;
     };
 
     /**
