@@ -163,217 +163,83 @@ void EventImpl::initEvent(const DOMString &eventTypeArg, bool canBubbleArg, bool
     m_cancelable = cancelableArg;
 }
 
-EventImpl::EventId EventImpl::typeToId(DOMString type)
-{
-    if (type == "DOMFocusIn")
-	return DOMFOCUSIN_EVENT;
-    else if (type == "DOMFocusOut")
-	return DOMFOCUSOUT_EVENT;
-    else if (type == "DOMActivate")
-	return DOMACTIVATE_EVENT;
-    else if (type == "click")
-	return CLICK_EVENT;
-    else if (type == "mousedown")
-	return MOUSEDOWN_EVENT;
-    else if (type == "mouseup")
-	return MOUSEUP_EVENT;
-    else if (type == "mouseover")
-	return MOUSEOVER_EVENT;
-    else if (type == "mousemove")
-	return MOUSEMOVE_EVENT;
-    else if (type == "mouseout")
-	return MOUSEOUT_EVENT;
-    else if (type == "onbeforecut")
-	return BEFORECUT_EVENT;
-    else if (type == "oncut")
-	return CUT_EVENT;
-    else if (type == "onbeforecopy")
-	return BEFORECOPY_EVENT;
-    else if (type == "oncopy")
-	return COPY_EVENT;
-    else if (type == "onbeforepaste")
-	return BEFOREPASTE_EVENT;
-    else if (type == "onpaste")
-	return PASTE_EVENT;
-    else if (type == "dragenter")
-	return DRAGENTER_EVENT;
-    else if (type == "dragover")
-	return DRAGOVER_EVENT;
-    else if (type == "dragleave")
-	return DRAGLEAVE_EVENT;
-    else if (type == "drop")
-	return DROP_EVENT;
-    else if (type == "dragstart")
-	return DRAGSTART_EVENT;
-    else if (type == "drag")
-	return DRAG_EVENT;
-    else if (type == "dragend")
-	return DRAGEND_EVENT;
-    else if (type == "selectstart")
-	return SELECTSTART_EVENT;
-    else if (type == "DOMSubtreeModified")
-	return DOMSUBTREEMODIFIED_EVENT;
-    else if (type == "DOMNodeInserted")
-	return DOMNODEINSERTED_EVENT;
-    else if (type == "DOMNodeRemoved")
-	return DOMNODEREMOVED_EVENT;
-    else if (type == "DOMNodeRemovedFromDocument")
-	return DOMNODEREMOVEDFROMDOCUMENT_EVENT;
-    else if (type == "DOMNodeInsertedIntoDocument")
-	return DOMNODEINSERTEDINTODOCUMENT_EVENT;
-    else if (type == "DOMAttrModified")
-	return DOMATTRMODIFIED_EVENT;
-    else if (type == "DOMCharacterDataModified")
-	return DOMCHARACTERDATAMODIFIED_EVENT;
-    else if (type == "load")
-	return LOAD_EVENT;
-    else if (type == "unload")
-	return UNLOAD_EVENT;
-    else if (type == "abort")
-	return ABORT_EVENT;
-    else if (type == "error")
-	return ERROR_EVENT;
-    else if (type == "select")
-	return SELECT_EVENT;
-    else if (type == "change")
-	return CHANGE_EVENT;
-    else if (type == "submit")
-	return SUBMIT_EVENT;
-    else if (type == "reset")
-	return RESET_EVENT;
-    else if (type == "focus")
-	return FOCUS_EVENT;
-    else if (type == "blur")
-	return BLUR_EVENT;
-    else if (type == "resize")
-	return RESIZE_EVENT;
-    else if (type == "scroll")
-	return SCROLL_EVENT;
-    else if (type == "contextmenu")
-	return CONTEXTMENU_EVENT;
-    else if (type == "keydown")
-	return KEYDOWN_EVENT;
-    else if (type == "keyup")
-	return KEYUP_EVENT;
+static const char * const eventNames[EventImpl::numEventIds] = {
+    0,
+    "DOMFocusIn",
+    "DOMFocusOut",
+    "DOMActivate",
+    "click",
+    "mousedown",
+    "mouseup",
+    "mouseover",
+    "mousemove",
+    "mouseout",
+    "onbeforecut",
+    "oncut",
+    "onbeforecopy",
+    "oncopy",
+    "onbeforepaste",
+    "onpaste",
+    "dragenter",
+    "dragover",
+    "dragleave",
+    "drop",
+    "dragstart",
+    "drag",
+    "dragend",
+    "selectstart",
+    "DOMSubtreeModified",
+    "DOMNodeInserted",
+    "DOMNodeRemoved",
+    "DOMNodeRemovedFromDocument",
+    "DOMNodeInsertedIntoDocument",
+    "DOMAttrModified",
+    "DOMCharacterDataModified",
+    "load",
+    "unload",
+    "abort",
+    "error",
+    "select",
+    "change",
+    "submit",
+    "reset",
+    "focus",
+    "blur",
+    "resize",
+    "scroll",
+    "contextmenu",
 #if APPLE_CHANGES
-    else if (type == "search")
-	return SEARCH_EVENT;
+    "search",
 #endif
-    else if (type == "input")
-        return INPUT_EVENT;
-    else if (type == "textInput")
-	return TEXTINPUT_EVENT;
-    else if (type == "readystatechange")
-	return KHTML_READYSTATECHANGE_EVENT;
-    // ignore: KHTML_DBLCLICK_EVENT
-    // ignore: KHTML_CLICK_EVENT
+    "input",
+    "keydown",
+    "keyup",
+    "textInput", // FIXME: is the capital I correct?
+    0, // KHTML_DBLCLICK_EVENT
+    0, // KHTML_CLICK_EVENT
+    0, // KHTML_DRAGDROP_EVENT
+    0, // KHTML_ERROR_EVENT
+    "keypress",
+    0, // KHTML_MOVE_EVENT
+    0, // KHTML_ORIGCLICK_MOUSEUP_EVENT
+    "readystatechange",
+    "mousewheel",
+    0, // horizontal mouse wheel
+};
+
+EventImpl::EventId EventImpl::typeToId(const DOMString &type)
+{
+    for (int i = 0; i < numEventIds; ++i) {
+        const char *n = eventNames[i];
+        if (n && type == n)
+            return static_cast<EventId>(i);
+    }
     return UNKNOWN_EVENT;
 }
 
-DOMString EventImpl::idToType(EventImpl::EventId id)
+DOMString EventImpl::idToType(EventId id)
 {
     switch (id) {
-	case DOMFOCUSIN_EVENT:
-	    return "DOMFocusIn";
-	case DOMFOCUSOUT_EVENT:
-	    return "DOMFocusOut";
-	case DOMACTIVATE_EVENT:
-	    return "DOMActivate";
-	case CLICK_EVENT:
-	    return "click";
-	case MOUSEDOWN_EVENT:
-	    return "mousedown";
-	case MOUSEUP_EVENT:
-	    return "mouseup";
-	case MOUSEOVER_EVENT:
-	    return "mouseover";
-	case MOUSEMOVE_EVENT:
-	    return "mousemove";
-	case MOUSEOUT_EVENT:
-	    return "mouseout";
-        case BEFORECUT_EVENT:
-            return "onbeforecut";
-	case CUT_EVENT:
-            return "oncut";
-	case BEFORECOPY_EVENT:
-            return "onbeforecopy";
-	case COPY_EVENT:
-            return "oncopy";
-	case BEFOREPASTE_EVENT:
-            return "onbeforepaste";
-	case PASTE_EVENT:
-            return "onpaste";
-	case DRAGENTER_EVENT:
-            return "dragenter";
-	case DRAGOVER_EVENT:
-            return "dragover";
-	case DRAGLEAVE_EVENT:
-            return "dragleave";
-	case DROP_EVENT:
-	    return "drop";
-	case DRAGSTART_EVENT:
-	    return "dragstart";
-	case DRAG_EVENT:
-	    return "drag";
-	case DRAGEND_EVENT:
-	    return "dragend";
-	case SELECTSTART_EVENT:
-	    return "selectstart";
-	case DOMSUBTREEMODIFIED_EVENT:
-	    return "DOMSubtreeModified";
-	case DOMNODEINSERTED_EVENT:
-	    return "DOMNodeInserted";
-	case DOMNODEREMOVED_EVENT:
-	    return "DOMNodeRemoved";
-	case DOMNODEREMOVEDFROMDOCUMENT_EVENT:
-	    return "DOMNodeRemovedFromDocument";
-	case DOMNODEINSERTEDINTODOCUMENT_EVENT:
-	    return "DOMNodeInsertedIntoDocument";
-	case DOMATTRMODIFIED_EVENT:
-	    return "DOMAttrModified";
-	case DOMCHARACTERDATAMODIFIED_EVENT:
-	    return "DOMCharacterDataModified";
-	case LOAD_EVENT:
-	    return "load";
-	case UNLOAD_EVENT:
-	    return "unload";
-	case ABORT_EVENT:
-	    return "abort";
-	case ERROR_EVENT:
-	    return "error";
-	case SELECT_EVENT:
-	    return "select";
-	case CHANGE_EVENT:
-	    return "change";
-	case SUBMIT_EVENT:
-	    return "submit";
-	case RESET_EVENT:
-	    return "reset";
-	case FOCUS_EVENT:
-	    return "focus";
-	case BLUR_EVENT:
-	    return "blur";
-	case RESIZE_EVENT:
-	    return "resize";
-	case SCROLL_EVENT:
-	    return "scroll";
-        case CONTEXTMENU_EVENT:
-            return "contextmenu";
-	case KEYDOWN_EVENT:
-            return "keydown";
-	case KEYUP_EVENT:
-            return "keyup";
-        case KEYPRESS_EVENT:
-            return "keypress";
-	case TEXTINPUT_EVENT:
-            return "textInput";
-#if APPLE_CHANGES
-        case SEARCH_EVENT:
-            return "search";
-#endif
-        case INPUT_EVENT:
-            return "input";
-	// khtml extensions
 	case KHTML_DBLCLICK_EVENT:
             return "dblclick";
 	case KHTML_CLICK_EVENT:
@@ -386,12 +252,12 @@ DOMString EventImpl::idToType(EventImpl::EventId id)
             return "khtml_move";
         case KHTML_ORIGCLICK_MOUSEUP_EVENT:
             return "khtml_origclick_mouseup_event";
-        case KHTML_READYSTATECHANGE_EVENT:
-	    return "readystatechange";
-	default:
-	    return DOMString();
-	    break;
+        default:
+            break;
     }
+    if (id >= numEventIds)
+        return DOMString();
+    return eventNames[id];
 }
 
 bool EventImpl::isUIEvent() const
@@ -424,6 +290,11 @@ bool EventImpl::isClipboardEvent() const
     return false;
 }
 
+bool EventImpl::isWheelEvent() const
+{
+    return false;
+}
+
 // -----------------------------------------------------------------------------
 
 UIEventImpl::UIEventImpl()
@@ -472,16 +343,71 @@ bool UIEventImpl::isUIEvent() const
 
 // -----------------------------------------------------------------------------
 
+// -----------------------------------------------------------------------------
+
+MouseRelatedEventImpl::MouseRelatedEventImpl()
+    : m_screenX(0), m_screenY(0), m_clientX(0), m_clientY(0), m_layerX(0), m_layerY(0)
+{
+}
+
+MouseRelatedEventImpl::MouseRelatedEventImpl(EventId _id,
+			       bool canBubbleArg,
+			       bool cancelableArg,
+			       AbstractViewImpl *viewArg,
+			       long detailArg,
+			       long screenXArg,
+			       long screenYArg,
+			       long clientXArg,
+			       long clientYArg,
+			       bool ctrlKeyArg,
+			       bool altKeyArg,
+			       bool shiftKeyArg,
+			       bool metaKeyArg)
+    : UIEventWithKeyStateImpl(_id, canBubbleArg, cancelableArg, viewArg, detailArg,
+        ctrlKeyArg, altKeyArg, shiftKeyArg, metaKeyArg)
+{
+    m_screenX = screenXArg;
+    m_screenY = screenYArg;
+    m_clientX = clientXArg;
+    m_clientY = clientYArg;
+    computeLayerPos();
+}
+
+void MouseRelatedEventImpl::computeLayerPos()
+{
+    m_layerX = m_clientX;
+    m_layerY = m_clientY;
+
+    DocumentImpl *doc = view()->document();
+
+    if (!doc || !doc->renderer()) {
+	return;
+    }
+
+    khtml::RenderObject::NodeInfo renderInfo(true, false);
+    doc->renderer()->layer()->hitTest(renderInfo, m_clientX, m_clientY);
+
+    NodeImpl *node = renderInfo.innerNonSharedNode();
+    while (node && !node->renderer()) {
+	node = node->parent();
+    }
+
+    if (!node) {
+	return;
+    }
+
+    node->renderer()->enclosingLayer()->updateLayerPosition();
+    
+    for (khtml::RenderLayer *layer = node->renderer()->enclosingLayer(); layer != NULL; layer = layer->parent()) {
+	m_layerX -= layer->xPos();
+	m_layerY -= layer->yPos();
+    }
+}
+
+// -----------------------------------------------------------------------------
+
 MouseEventImpl::MouseEventImpl()
 {
-    m_screenX = 0;
-    m_screenY = 0;
-    m_clientX = 0;
-    m_clientY = 0;
-    m_ctrlKey = false;
-    m_altKey = false;
-    m_shiftKey = false;
-    m_metaKey = false;
     m_button = 0;
     m_relatedTarget = 0;
     m_clipboard = 0;
@@ -503,16 +429,10 @@ MouseEventImpl::MouseEventImpl(EventId _id,
 			       unsigned short buttonArg,
 			       NodeImpl *relatedTargetArg,
                                ClipboardImpl *clipboardArg)
-		   : UIEventImpl(_id,canBubbleArg,cancelableArg,viewArg,detailArg)
+    : MouseRelatedEventImpl(_id, canBubbleArg, cancelableArg, viewArg, detailArg,
+        screenXArg, screenYArg, clientXArg, clientYArg,
+        ctrlKeyArg, altKeyArg, shiftKeyArg, metaKeyArg)
 {
-    m_screenX = screenXArg;
-    m_screenY = screenYArg;
-    m_clientX = clientXArg;
-    m_clientY = clientYArg;
-    m_ctrlKey = ctrlKeyArg;
-    m_altKey = altKeyArg;
-    m_shiftKey = shiftKeyArg;
-    m_metaKey = metaKeyArg;
     m_button = buttonArg;
     m_relatedTarget = relatedTargetArg;
     if (m_relatedTarget)
@@ -520,38 +440,6 @@ MouseEventImpl::MouseEventImpl(EventId _id,
     m_clipboard = clipboardArg;
     if (m_clipboard)
 	m_clipboard->ref();
-    computeLayerPos();
-}
-
-void MouseEventImpl::computeLayerPos()
-{
-    m_layerX = m_clientX;
-    m_layerY = m_clientY;
-
-    DocumentImpl *doc = view()->document();
-
-    if (!doc) {
-	return;
-    }
-
-    khtml::RenderObject::NodeInfo renderInfo(true, false);
-    doc->renderer()->layer()->nodeAtPoint(renderInfo, m_clientX, m_clientY);
-
-    NodeImpl *node = renderInfo.innerNonSharedNode();
-    while (node && !node->renderer()) {
-	node = node->parent();
-    }
-
-    if (!node) {
-	return;
-    }
-
-    node->renderer()->enclosingLayer()->updateLayerPosition();
-    
-    for (khtml::RenderLayer *layer = node->renderer()->enclosingLayer(); layer != NULL; layer = layer->parent()) {
-	m_layerX -= layer->xPos();
-	m_layerY -= layer->yPos();
-    }
 }
 
 MouseEventImpl::~MouseEventImpl()
@@ -618,16 +506,16 @@ KeyboardEventImpl::KeyboardEventImpl()
   m_keyEvent = 0;
   m_keyIdentifier = 0;
   m_keyLocation = KeyboardEvent::DOM_KEY_LOCATION_STANDARD;
-  m_ctrlKey = false;
-  m_shiftKey = false;
-  m_altKey = false;
-  m_metaKey = false;
   m_altGraphKey = false;
 }
 
 KeyboardEventImpl::KeyboardEventImpl(QKeyEvent *key, AbstractViewImpl *view)
-  : UIEventImpl(key->type() == QEvent::KeyRelease ? KEYUP_EVENT : key->isAutoRepeat() ? KEYPRESS_EVENT : KEYDOWN_EVENT,
-                true,true,view,0)
+  : UIEventWithKeyStateImpl(key->type() == QEvent::KeyRelease ? KEYUP_EVENT : key->isAutoRepeat() ? KEYPRESS_EVENT : KEYDOWN_EVENT,
+    true, true, view, 0,
+    key->state() & Qt::ControlButton,
+    key->state() & Qt::AltButton,
+    key->state() & Qt::ShiftButton,
+    key->state() & Qt::MetaButton)
 {
 #if APPLE_CHANGES
     m_keyEvent = new QKeyEvent(*key);
@@ -646,10 +534,6 @@ KeyboardEventImpl::KeyboardEventImpl(QKeyEvent *key, AbstractViewImpl *view)
 
     int keyState = key->state();
 
-    m_ctrlKey = keyState & Qt::ControlButton;
-    m_shiftKey = keyState & Qt::ShiftButton;
-    m_altKey = keyState & Qt::AltButton;
-    m_metaKey = keyState & Qt::MetaButton;
     m_altGraphKey = false; // altGraphKey is not supported by Qt.
     
     // Note: we only support testing for num pad
@@ -663,21 +547,17 @@ KeyboardEventImpl::KeyboardEventImpl(EventId _id,
                                         const DOMString &keyIdentifierArg, 
                                         unsigned long keyLocationArg, 
                                         bool ctrlKeyArg, 
-                                        bool shiftKeyArg, 
                                         bool altKeyArg, 
+                                        bool shiftKeyArg, 
                                         bool metaKeyArg, 
                                         bool altGraphKeyArg)
-  : UIEventImpl(_id,canBubbleArg,cancelableArg,viewArg,0)
+  : UIEventWithKeyStateImpl(_id, canBubbleArg, cancelableArg, viewArg, 0, ctrlKeyArg, altKeyArg, shiftKeyArg, metaKeyArg)
 {
     m_keyEvent = 0;
     m_keyIdentifier = keyIdentifierArg.implementation();
     if (m_keyIdentifier)
         m_keyIdentifier->ref();
     m_keyLocation = keyLocationArg;
-    m_ctrlKey = ctrlKeyArg;
-    m_shiftKey = shiftKeyArg;
-    m_altKey = altKeyArg;
-    m_metaKey = metaKeyArg;
     m_altGraphKey = altGraphKeyArg;
 }
 
@@ -695,8 +575,8 @@ void KeyboardEventImpl::initKeyboardEvent(const DOMString &typeArg,
                         const DOMString &keyIdentifierArg, 
                         unsigned long keyLocationArg, 
                         bool ctrlKeyArg, 
-                        bool shiftKeyArg, 
                         bool altKeyArg, 
+                        bool shiftKeyArg, 
                         bool metaKeyArg, 
                         bool altGraphKeyArg)
 {
@@ -865,6 +745,25 @@ bool ClipboardEventImpl::isClipboardEvent() const
 
 // -----------------------------------------------------------------------------
 
+WheelEventImpl::WheelEventImpl() : m_horizontal(false), m_wheelDelta(0)
+{
+}
+
+WheelEventImpl::WheelEventImpl(bool h, long d, AbstractViewImpl *v,
+    long sx, long sy, long cx, long cy, bool ctrl, bool alt, bool shift, bool meta)
+    : MouseRelatedEventImpl(h ? HORIZONTALMOUSEWHEEL_EVENT : MOUSEWHEEL_EVENT,
+        true, true, v, 0, sx, sy, cx, cy, ctrl, alt, shift, meta)
+    , m_horizontal(h), m_wheelDelta(d)
+{
+}
+
+bool WheelEventImpl::isWheelEvent() const
+{
+    return true;
+}
+
+// -----------------------------------------------------------------------------
+
 RegisteredEventListener::RegisteredEventListener(EventImpl::EventId _id, EventListener *_listener, bool _useCapture)
 {
     id = _id;
@@ -886,10 +785,6 @@ bool RegisteredEventListener::operator==(const RegisteredEventListener &other)
 
 // -----------------------------------------------------------------------------
 
-ClipboardImpl::ClipboardImpl()
-{
-}
-
 ClipboardImpl::~ClipboardImpl()
 {
 }
