@@ -4,7 +4,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2003 Apple Computer, Inc.
+ * Copyright (C) 2004 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -30,12 +30,11 @@
 #include "dom/dom_node.h"
 #include "misc/helper.h"
 #include "misc/shared.h"
-
-// The namespace used for XHTML elements
-#define XHTML_NAMESPACE "http://www.w3.org/1999/xhtml"
+#include "dom_atomicstring.h"
 
 class QPainter;
 template <class type> class QPtrList;
+template <class type> class QPtrDict;
 class KHTMLView;
 class RenderArena;
 class QRect;
@@ -43,6 +42,7 @@ class QMouseEvent;
 class QKeyEvent;
 class QTextStream;
 class QStringList;
+class QWheelEvent;
 
 namespace khtml {
     class RenderObject;
@@ -51,12 +51,29 @@ namespace khtml {
 
 namespace DOM {
 
-class NodeListImpl;
-class NamedNodeMapImpl;
 class DocumentImpl;
-class CSSStyleDeclarationImpl;
-class RegisteredEventListener;
+class ElementImpl;
 class EventImpl;
+class NodeListImpl;
+class RegisteredEventListener;
+
+// The namespace used for XHTML elements
+#define XHTML_NAMESPACE "http://www.w3.org/1999/xhtml"
+
+const Q_UINT16 noNamespace = 0;
+const Q_UINT16 anyNamespace = 1;
+const Q_UINT16 xhtmlNamespace = 2;
+const Q_UINT16 anyLocalName = 0;
+
+const Q_UINT32 namespaceMask = 0xFFFF0000U;
+const Q_UINT32 localNameMask = 0x0000FFFFU;
+
+inline Q_UINT16 namespacePart(Q_UINT32 i) { return i >> 16; }
+inline Q_UINT16 localNamePart(Q_UINT32 i) { return i; }
+inline Q_UINT32 makeId(Q_UINT16 n, Q_UINT16 l) { return (n << 16) | l; }
+
+// Can't use makeId here because it results in an "initroutine".
+const Q_UINT32 anyQName = anyNamespace << 16 | anyLocalName;
 
 class DocumentPtr : public khtml::Shared<DocumentPtr>
 {
@@ -93,6 +110,7 @@ public:
     virtual NodeImpl *replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
     virtual NodeImpl *removeChild ( NodeImpl *oldChild, int &exceptioncode );
     virtual NodeImpl *appendChild ( NodeImpl *newChild, int &exceptioncode );
+    virtual void remove(int &exceptioncode);
     virtual bool hasChildNodes (  ) const;
     virtual NodeImpl *cloneNode ( bool deep ) = 0;
     virtual DOMString localName() const;
@@ -100,6 +118,8 @@ public:
     virtual void setPrefix(const DOMString &_prefix, int &exceptioncode );
     void normalize ();
 
+    NodeImpl *lastDescendent() const;
+
     // Other methods (not part of DOM)
     virtual bool isElementNode() const { return false; }
     virtual bool isHTMLElement() const { return false; }
@@ -107,6 +127,8 @@ public:
     virtual bool isTextNode() const { return false; }
     virtual bool isDocumentNode() const { return false; }
     virtual bool isXMLElementNode() const { return false; }
+    bool isBlockFlow() const;
+    bool isBlockFlowOrTable() const;
     
     // Used by <form> elements to indicate a malformed state of some kind, typically
     // used to keep from applying the bottom margin of the form.
@@ -123,13 +145,48 @@ public:
     virtual void setFirstChild(NodeImpl *child);
     virtual void setLastChild(NodeImpl *child);
 
+    bool isAtomicNode() const;
+    NodeImpl *previousNodeConsideringAtomicNodes() const;
+    NodeImpl *nextNodeConsideringAtomicNodes() const;
+    
+    /** (Not part of the official DOM)
+     * Returns the next leaf node.
+     *
+     * Using this function delivers leaf nodes as if the whole DOM tree
+     * were a linear chain of its leaf nodes.
+     * @return next leaf node or 0 if there are no more.
+     */
+    NodeImpl *nextLeafNode() const;
+
+    /** (Not part of the official DOM)
+     * Returns the previous leaf node.
+     *
+     * Using this function delivers leaf nodes as if the whole DOM tree
+     * were a linear chain of its leaf nodes.
+     * @return previous leaf node or 0 if there are no more.
+     */
+    NodeImpl *previousLeafNode() const;
+
+    bool isEditableBlock() const;
+    ElementImpl *enclosingBlockFlowElement() const;
+    ElementImpl *enclosingBlockFlowOrTableElement() const;
+    ElementImpl *enclosingInlineElement() const;
+    ElementImpl *rootEditableElement() const;
+    
+    bool inSameRootEditableElement(NodeImpl *);
+    bool inSameContainingBlockFlowElement(NodeImpl *);
+    
     // used by the parser. Doesn't do as many error checkings as
     // appendChild(), and returns the node into which will be parsed next.
     virtual NodeImpl *addChild(NodeImpl *newChild);
+    
+    // called by the parser when this element's close tag is reached,
+    // signalling that all child tags have been parsed and added.
+    // This is only needed for <applet> and <object> elements, which can't lay themselves out
+    // until they know all of their nested <param>s. [3603191, 4040848]
+    virtual void closeRenderer() { }
 
     typedef Q_UINT32 Id;
-    static const Q_UINT32 IdNSMask;
-    static const Q_UINT32 IdLocalMask;
     // id() is used to easily and exactly identify a node. It
     // is optimized for quick comparison and low memory consumption.
     // its value depends on the owner document of the node and is
@@ -145,7 +202,9 @@ public:
     //                 the lower 16 bit identify the local part of the
     //                 qualified element name.
     virtual Id id() const { return 0; };
-
+#if APPLE_CHANGES
+    Id identifier() const;
+#endif
     enum MouseEventType {
         MousePress,
         MouseRelease,
@@ -194,7 +253,7 @@ public:
     void setInDocument(bool b=true) { m_inDocument = b; }
     virtual void setFocus(bool b=true) { m_focused = b; }
     virtual void setActive(bool b=true) { m_active = b; }
-    virtual void setChanged(bool b=true);
+    void setChanged(bool b=true);
 
     unsigned short tabIndex() const { return m_tabIndex; }
     void setTabIndex(unsigned short _tabIndex) { m_tabIndex = _tabIndex; }
@@ -207,10 +266,8 @@ public:
     virtual bool isMouseFocusable() const;
     
     virtual bool isInline() const;
-    virtual QString toHTML() const;
-    QString recursive_toHTML(bool start = false) const;
-
-    virtual void getCursor(int offset, int &_x, int &_y, int &height);
+    
+    virtual bool isContentEditable() const;
     virtual QRect getRect() const;
 
     enum StyleChange { NoChange, NoInherit, Inherit, Detach, Force };
@@ -229,6 +286,7 @@ public:
     void removeHTMLEventListener(int id);
     void setHTMLEventListener(int id, EventListener *listener);
     EventListener *getHTMLEventListener(int id);
+    void removeAllEventListeners();
 
     bool dispatchEvent(EventImpl *evt, int &exceptioncode, bool tempEvent = false);
     bool dispatchGenericEvent( EventImpl *evt, int &exceptioncode);
@@ -236,8 +294,9 @@ public:
     bool dispatchWindowEvent(int _id, bool canBubbleArg, bool cancelableArg);
     bool dispatchMouseEvent(QMouseEvent *e, int overrideId = 0, int overrideDetail = 0);
     bool dispatchUIEvent(int _id, int detail = 0);
-    bool dispatchSubtreeModifiedEvent();
+    bool dispatchSubtreeModifiedEvent(bool childrenChanged = true);
     bool dispatchKeyEvent(QKeyEvent *key);
+    void dispatchWheelEvent(QWheelEvent *);
 
     void handleLocalEvents(EventImpl *evt, bool useCapture);
 
@@ -254,7 +313,7 @@ public:
 
     virtual bool isReadOnly();
     virtual bool childTypeAllowed( unsigned short /*type*/ ) { return false; }
-    virtual unsigned long childNodeCount();
+    virtual unsigned long childNodeCount() const;
     virtual NodeImpl *childNode(unsigned long index);
 
     /**
@@ -268,7 +327,10 @@ public:
      *
      * see @ref traversePreviousNode()
      */
-    NodeImpl *traverseNextNode(NodeImpl *stayWithin = 0) const;
+    NodeImpl *traverseNextNode(const NodeImpl *stayWithin = 0) const;
+    
+    /* Like traverseNextNode, but skips children and starts with the next sibling. */
+    NodeImpl *traverseNextSibling(const NodeImpl *stayWithin = 0) const;
 
     /**
      * Does a reverse pre-order traversal to find the node that comes before the current one in document order
@@ -277,8 +339,15 @@ public:
      */
     NodeImpl *traversePreviousNode() const;
 
+    /* Like traversePreviousNode, but visits nodes before their children. */
+    NodeImpl *traversePreviousNodePostOrder(const NodeImpl *stayWithin = 0) const;
+
     DocumentPtr *docPtr() const { return document; }
 
+    NodeImpl *previousEditable() const;
+    NodeImpl *nextEditable() const;
+    //bool isEditable() const;
+
     khtml::RenderObject *renderer() const { return m_render; }
     khtml::RenderObject *nextRenderer();
     khtml::RenderObject *previousRenderer();
@@ -286,8 +355,17 @@ public:
     
     void checkSetPrefix(const DOMString &_prefix, int &exceptioncode);
     void checkAddChild(NodeImpl *newChild, int &exceptioncode);
-    bool isAncestor( NodeImpl *other );
+    bool isAncestor(const NodeImpl *) const;
     virtual bool childAllowed( NodeImpl *newChild );
+
+    virtual long maxOffset() const;
+    virtual long caretMinOffset() const;
+    virtual long caretMaxOffset() const;
+    virtual unsigned long caretMaxRenderedOffset() const;
+
+    virtual long previousOffset (long current) const;
+    virtual long nextOffset (long current) const;
+    
 #ifndef NDEBUG
     virtual void dump(QTextStream *stream, QString ind = "") const;
 #endif
@@ -308,8 +386,6 @@ public:
      */
     virtual void detach();
 
-    void closeRenderer();
-
     void createRendererIfNeeded();
     virtual khtml::RenderStyle *styleForRenderer(khtml::RenderObject *parent);
     virtual bool rendererIsNeeded(khtml::RenderStyle *);
@@ -368,11 +444,23 @@ public:
 
     /**
      * Notifies the node that it's list of children have changed (either by adding or removing child nodes), or a child
-     * node that is of the type CDATA_SECTION_NODE, TEXT_NODE or COMMENT_NODE has changed it's value.
+     * node that is of the type CDATA_SECTION_NODE, TEXT_NODE or COMMENT_NODE has changed its value.
      */
     virtual void childrenChanged();
 
     virtual DOMString toString() const = 0;
+    
+#ifndef NDEBUG
+    virtual void formatForDebugger(char *buffer, unsigned length) const;
+
+    void displayNode(const char *prefix="");
+    void displayTree();
+#endif
+
+    void registerNodeList(NodeListImpl *list);
+    void unregisterNodeList(NodeListImpl *list);
+    void notifyNodeListsSubtreeModified();
+    void notifyLocalNodeListsSubtreeModified();
 
 private: // members
     DocumentPtr *document;
@@ -381,6 +469,7 @@ private: // members
 protected:
     khtml::RenderObject *m_render;
     QPtrList<RegisteredEventListener> *m_regdListeners;
+    QPtrDict<NodeListImpl> *m_nodeLists;
 
     unsigned short m_tabIndex : 15;
     bool m_hasTabIndex  : 1;
@@ -399,9 +488,8 @@ protected:
     bool m_active : 1;
     bool m_styleElement : 1; // contains stylesheet text
     bool m_implicit : 1; // implicitely generated by the parser
-    bool m_rendererNeedsClose : 1;
 
-    // 1 bit unused
+    // 3 bits unused
 };
 
 // this is the full Node Implementation with parents and children.
@@ -439,9 +527,12 @@ public:
 
     virtual void setFocus(bool=true);
     virtual void setActive(bool=true);
-    virtual unsigned long childNodeCount();
+    virtual unsigned long childNodeCount() const;
     virtual NodeImpl *childNode(unsigned long index);
 
+    virtual void insertedIntoDocument();
+    virtual void removedFromDocument();
+    
     // check for being (grand-..)father:
     bool checkNoOwner( NodeImpl *other, int &exceptioncode );
 
@@ -470,29 +561,39 @@ class NodeImpl;
 class NodeListImpl : public khtml::Shared<NodeListImpl>
 {
 public:
-    virtual ~NodeListImpl() {}
+    NodeListImpl( NodeImpl *_rootNode );
+    virtual ~NodeListImpl();
 
     // DOM methods & attributes for NodeList
-    virtual unsigned long length() const;
-    virtual NodeImpl *item ( unsigned long index ) const;
+    virtual unsigned long length() const = 0;
+    virtual NodeImpl *item ( unsigned long index ) const = 0;
+    virtual NodeImpl *itemById ( const DOMString & elementId ) const;
 
     // Other methods (not part of DOM)
 
+    void rootNodeSubtreeModified();
+
 #if APPLE_CHANGES
     static NodeList createInstance(NodeListImpl *impl);
 #endif
 protected:
     // helper functions for searching all ElementImpls in a tree
-    unsigned long recursiveLength(NodeImpl *start) const;
-    NodeImpl *recursiveItem ( NodeImpl *start, unsigned long &offset ) const;
+    unsigned long recursiveLength(NodeImpl *start = 0) const;
+    NodeImpl *recursiveItem ( unsigned long offset, NodeImpl *start = 0 ) const;
     virtual bool nodeMatches( NodeImpl *testNode ) const = 0;
+
+    NodeImpl *rootNode;
+    mutable int cachedLength;
+    mutable NodeImpl *lastItem;
+    mutable unsigned long lastItemOffset;
+    mutable bool isLengthCacheValid : 1;
+    mutable bool isItemCacheValid : 1;
 };
 
 class ChildNodeListImpl : public NodeListImpl
 {
 public:
     ChildNodeListImpl( NodeImpl *n);
-    virtual ~ChildNodeListImpl();
 
     // DOM methods overridden from  parent classes
 
@@ -501,8 +602,6 @@ public:
 
 protected:
     virtual bool nodeMatches( NodeImpl *testNode ) const;
-
-    NodeImpl *refNode;
 };
 
 
@@ -513,7 +612,6 @@ class TagNodeListImpl : public NodeListImpl
 {
 public:
     TagNodeListImpl( NodeImpl *n, NodeImpl::Id tagId, NodeImpl::Id tagIdMask );
-    virtual ~TagNodeListImpl();
 
     // DOM methods overridden from  parent classes
 
@@ -525,7 +623,6 @@ public:
 protected:
     virtual bool nodeMatches( NodeImpl *testNode ) const;
 
-    NodeImpl *refNode;
     NodeImpl::Id m_id;
     NodeImpl::Id m_idMask;
 };
@@ -538,7 +635,6 @@ class NameNodeListImpl : public NodeListImpl
 {
 public:
     NameNodeListImpl( NodeImpl *doc, const DOMString &t );
-    virtual ~NameNodeListImpl();
 
     // DOM methods overridden from  parent classes
 
@@ -550,7 +646,6 @@ public:
 protected:
     virtual bool nodeMatches( NodeImpl *testNode ) const;
 
-    NodeImpl *refNode;
     DOMString nodeName;
 };
 
