@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Apple Computer, Inc.  All rights reserved.
+ * Copyright (C) 2004 Apple Computer, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,7 +30,6 @@
 #include "KWQSize.h"
 #include "KWQString.h"
 #include "KWQFont.h"
-#include "KWQFontInfo.h"
 
 class QFontMetricsPrivate;
 
@@ -61,15 +60,13 @@ public:
                      int letterSpacing, int wordSpacing, bool smallCaps) const;
     float floatCharacterWidths(const QChar *, int slen, int pos, int len, int toAdd, float *buffer,
                                int letterSpacing, int wordSpacing, bool smallCaps) const;
-    int checkSelectionPoint (QChar *s, int slen, int pos, int len, int toAdd, int letterSpacing, int wordSpacing, bool smallCaps, int x, bool reversed) const;
+    int checkSelectionPoint (QChar *s, int slen, int pos, int len, int toAdd, int letterSpacing, int wordSpacing, bool smallCaps, int x, bool reversed, bool includePartialGlyphs) const;
 
     QRect boundingRect(const QString &, int len=-1) const;
     QRect boundingRect(int, int, int, int, int, const QString &) const;
 
     QSize size(int, const QString &) const;
 
-    int rightBearing(QChar) const;
-    int leftBearing(QChar) const;
     int baselineOffset() const { return ascent(); }
     
 private:
