@@ -25,6 +25,7 @@
 
 #import "KWQKJavaAppletWidget.h"
 
+#import "dom_docimpl.h"
 #import "KHTMLView.h"
 #import "KWQExceptions.h"
 #import "KWQKJavaAppletContext.h"
@@ -34,7 +35,8 @@ KJavaAppletWidget::KJavaAppletWidget(const QSize &size, KJavaAppletContext *c, c
 {
     KWQ_BLOCK_EXCEPTIONS;
     
-    NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
+    NSMutableArray *attributeNames = [[NSMutableArray alloc] init];
+    NSMutableArray *attributeValues = [[NSMutableArray alloc] init];
     QMapConstIterator<QString, QString> it = args.begin();
     QMapConstIterator<QString, QString> end = args.end();
     QString baseURLString;
@@ -42,17 +44,21 @@ KJavaAppletWidget::KJavaAppletWidget(const QSize &size, KJavaAppletContext *c, c
         if (it.key().lower() == "baseurl") {
             baseURLString = it.data();
         }
-        [attributes setObject:it.data().getNSString() forKey:it.key().getNSString()];
+        [attributeNames addObject:it.key().getNSString()];
+        [attributeValues addObject:it.data().getNSString()];
         ++it;
     }
     
-    KHTMLPart *part = c->part();
-    KURL baseURL = baseURLString.isNull() ? part->baseURL() : KURL(baseURLString);
-    
-    setView([KWQ(part)->bridge() viewForJavaAppletWithFrame:NSMakeRect(0, 0, size.width(), size.height())
-                                                 attributes:attributes
-                                                    baseURL:baseURL.getNSURL()]);
-    [attributes release];
+    KWQKHTMLPart *part = KWQ(c->part());
+    if (baseURLString.isEmpty()) {
+        baseURLString = part->xmlDocImpl()->baseURL();
+    }
+    setView([part->bridge() viewForJavaAppletWithFrame:NSMakeRect(0, 0, size.width(), size.height())
+                                        attributeNames:attributeNames
+                                       attributeValues:attributeValues
+                                               baseURL:part->completeURL(baseURLString).getNSURL()]);
+    [attributeNames release];
+    [attributeValues release];
     part->view()->addChild(this);
     
     KWQ_UNBLOCK_EXCEPTIONS;
